import os
import shutil
from jinja2 import Template

from pagekey_docgen.config import PageKeySite


def get_files_list(path: str):
    """Walk directory and get all files recursively.
    
    Args:
      path: Directory path to walk.
    """
    result = []
    for root, dirs, files in os.walk(path):
        for cur_file in files:
            cur_file_path = os.path.relpath(os.path.join(root, cur_file))
            result.append(cur_file_path)
    return result

def create_output_directory():
    """Create output directory to copy the generated site into."""
    os.makedirs('build')

def remove_output_directory():
    """Remove the output directory recursively.
    
    Clean the project.
    """
    if os.path.exists('build'):
        shutil.rmtree('build')

def render_file(path: str):
    """Render a docs file to the final HTML site.

    Args:
      path: Path to file to be rendered.
    """
    dirname = os.path.dirname(path)
    if len(dirname) < 1:
        # File is at the top-level of the repo - keep it simple
        dest_dir_relpath = os.path.join('build', 'sphinx')
    else:
        # Handle nested files
        src_dir_relpath = os.path.relpath(os.path.dirname(path))
        dest_dir_relpath = os.path.join('build', 'sphinx', src_dir_relpath)
    # Create directories containing this file if not exists
    os.makedirs(dest_dir_relpath, exist_ok=True)
    # Copy the file over
    # TODO / NOTE: eventually this will do templating too
    shutil.copy(path, dest_dir_relpath)

def get_repo_root(cur_file=__file__):
    """Get root directory of installed package.
    
    Useful for copying files within the package.
    
    Args:
      cur_file: The value of __file__. Included as kwarg for testing purposes.  
    """
    return os.path.dirname(cur_file)

def render_template(filename: str, config: PageKeySite):
    """Render a template file.
    
    Args:
      filename: Name of file within templates directory.
    """
    repo_root = get_repo_root()
    src_path = os.path.join(repo_root, 'templates', filename)
    dest_path = os.path.join('build', 'sphinx', os.path.basename(filename))
    if not os.path.exists('build/sphinx'):
        os.makedirs('build/sphinx')
    file_contents = get_file_as_string(src_path)
    template = Template(file_contents)
    output_string = template.render(config=config)
    write_string_to_file(dest_path, output_string)

def get_file_as_string(filename: str):
    with open(filename, 'r') as file:
        return file.read()

def write_string_to_file(filename: str, data: str):
    with open(filename, 'w') as file:
        file.write(data)



import argparse
import os
import shutil
import sys
from typing import List
from pagekey_docgen.config import load_config

from pagekey_docgen.core import (
    get_file_as_string,
    get_files_list,
    remove_output_directory,
    create_output_directory,
    render_file,
    render_template,
)


class DocsDirNotFoundException(Exception):
    pass

def main(args_list: List[str] = sys.argv[1:]):
    parser = argparse.ArgumentParser(prog='pagekey-docgen', description='Generate documentation site from directory.')
    parser.add_argument('docs_dir', metavar='docs_dir', type=str, help='The path to the documents directory')

    args = parser.parse_args(args_list)
    
    if not os.path.exists(args.docs_dir):
        raise DocsDirNotFoundException()
    
    remove_output_directory()
    create_output_directory()

    files = get_files_list(args.docs_dir)

    # Grab config
    config_path = os.path.join(args.docs_dir, 'site.yaml')
    config_raw = get_file_as_string(config_path)
    config = load_config(config_raw)

    # Render templates
    for template in ['Makefile', 'make.bat', 'conf.py']:
        render_template(template, config)

    # Render source files
    for cur_file in files:
        render_file(cur_file)

    # Generate the Sphinx site
    os.chdir('build/sphinx')
    os.system('make html')

    # Move the generated site to the top level of the build directory
    shutil.move('_build/html', '..')



import yaml
from pydantic import BaseModel


class PageKeySite(BaseModel):
    project: str
    copyright: str
    author: str
    release: str
    package: str

def load_config(yaml_config: str):
    parsed_config = yaml.safe_load(yaml_config)
    site_config = PageKeySite(**parsed_config)
    return site_config



"""Template for Sphinx configuration file."""
# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = '{{ config.project }}'
copyright = '2024, {{ config.author }}'
author = '{{ config.author }}'
release = '{{ config.release }}'

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration

extensions = [
    'myst_parser',
    'sphinx.ext.autodoc',
    'autoapi.extension',
    'sphinxcontrib.mermaid',
]

templates_path = ['_templates']
exclude_patterns = ['_build', 'Thumbs.db', '.DS_Store']



# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output

html_theme = 'sphinx_rtd_theme'
html_static_path = ['_static']


# Add '.md' to source_suffix
source_suffix = {
    '.rst': 'restructuredtext',
    '.md': 'markdown',
}

autoapi_dirs = [
    '{{ config.package }}'
]
